#include "stdafx.h"
#include "OfficeClipHelperFactory.h"
#include "OfficeClipHelper.h"
#include <string>
#include <sstream>
#include <vector>
#include <map>

// Massively OTT but I expect to add more versions.
officefuncs findpf(std::wstring & ver)
{
	USES_CONVERSION;
	std::map<std::wstring, officefuncs> pfm;
	// yes, this is basically as evil as you can get.
	// Unfortunately the exported symbols from MSO.dll just aren't useful enough.
	pfm[L"11.0.5606.0"]		= officefuncs((clearclipfunc_f)0x31175784, (getcountfunc_f)0x30e87f74);
	pfm[L"11.0.6360.0"]		= officefuncs((clearclipfunc_f)0x3117941c, (getcountfunc_f)0x30e812cd);
	pfm[L"11.0.8172.0"]		= officefuncs((clearclipfunc_f)0x3118785a, (getcountfunc_f)0x30fd6a52);
	pfm[L"12.0.6245.1000"]	= officefuncs((clearclipfunc_f)0x32f69394, (getcountfunc_f)0x32cacf68);
	std::map<std::wstring, officefuncs>::iterator mit = pfm.find(ver);
	if (mit == pfm.end())
	{
		std::wstringstream ss;
		ss << "Unknown version of MSOffice [" << ver << "] - known versions are [";
		for (mit = pfm.begin(); mit != pfm.end(); ++mit)
		{
			ss << ((mit == pfm.begin()) ? "" : ", ") << mit->first;
		}
		ss << "]";
		throw std::runtime_error(T2A(ss.str().c_str()));
	}
	return mit->second;
}

// COfficeClipHelperFactory

STDMETHODIMP COfficeClipHelperFactory::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IOfficeClipHelperFactory
	};

	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}


int getversioninfo(std::wstring & fname, std::wstring & tgt, unsigned long & hw, unsigned long & lw)
{
	DWORD dwHandle, dwLen;
	UINT BufLen;
	VS_FIXEDFILEINFO *pFileInfo;

	dwLen = GetFileVersionInfoSize (fname.c_str(), &dwHandle);
	if (!dwLen)
		{ return 0; }
	std::vector<wchar_t> lpData(dwLen);
	if (!GetFileVersionInfo (fname.c_str(), dwHandle, dwLen, &lpData[0]))	 
		{ return 0; }
	if (!VerQueryValue (&lpData[0], L"\\", (LPVOID*) &pFileInfo, (PUINT) &BufLen)) 
		{ return 0; }
	
	std::wstringstream ss;
	ss << HIWORD(pFileInfo->dwFileVersionMS) << L"." <<
		  LOWORD(pFileInfo->dwFileVersionMS) << L"." <<
		  HIWORD(pFileInfo->dwFileVersionLS) << L"." <<
		  LOWORD(pFileInfo->dwFileVersionLS);

	tgt = ss.str();
	hw = pFileInfo->dwFileVersionMS;
	lw = pFileInfo->dwFileVersionLS;
	return 1;
}

STDMETHODIMP COfficeClipHelperFactory::get_OfficeClipHelper(IOfficeClipHelper ** pVal)
{
	if (!pVal) return E_POINTER;
	COM_FUN_BEGIN;

	USES_CONVERSION;
	HMODULE msoHandle = GetModuleHandleA("MSO.DLL");
	if (msoHandle <= 0) return Error("Can't get module handle for MSO.DLL");
	wchar_t path[MAX_PATH];
	if (!GetModuleFileName(msoHandle, path, sizeof(path))) return Error("Can't get path for MSO.DLL");

	std::wstring spath(path);
	std::wstring version;
	unsigned long hw = 0, lw = 0;
	if (!getversioninfo(spath, version, hw, lw)) return Error("Can't get version info for MSO.DLL");

	CComPtr<IOfficeClipHelper> pchelper;
	COfficeClipHelper::CreateInstance(&pchelper);
	COfficeClipHelper * pch = static_cast<COfficeClipHelper *>(pchelper.p);
	if (!pch) return Error("Unhandled error");
	pch->initialize(findpf(version), CComBSTR(version.c_str()), msoHandle);
	
	*pVal = pchelper.Detach();

	COM_FUN_END;
	return S_OK;
}
